import os

import trio
import pytest_trio

from trio_bybit.streams import BybitSocketManager


async def pub_stream_check(socket):
    async for msg in socket.get_next_message():
        assert "ts" in msg
        assert "type" in msg
        assert "data" in msg


async def pub_unsub(socket):
    unsub = {
        "op": "unsubscribe",
        "args": ["orderbook.1.BTCUSDT", "publicTrade.BTCUSDT"],
    }
    await socket.subscribe(unsub)


async def test_public_stream():
    socket = BybitSocketManager()
    async with socket.connect():
        subscription = {
            "op": "subscribe",
            "args": ["orderbook.1.BTCUSDT", "publicTrade.BTCUSDT"],
        }
        await socket.subscribe(subscription)

        with trio.move_on_after(12):
            async with trio.open_nursery() as nursery:
                nursery.start_soon(pub_stream_check, socket)
                await trio.sleep(10)
                nursery.start_soon(pub_unsub, socket)


async def test_public_linear_stream():
    socket = BybitSocketManager(endpoint="linear")
    async with socket.connect():
        subscription = {
            "op": "subscribe",
            "args": ["orderbook.1.BTCUSDT", "publicTrade.BTCUSDT"],
        }
        await socket.subscribe(subscription)

        count = 0
        async for msg in socket.get_next_message():
            count += 1
            if msg.get("topic") == "orderbook.1.BTCUSDT":
                assert "topic" in msg
                assert "type" in msg
                assert "data" in msg
                assert "s" in msg["data"]
                assert "a" in msg["data"]
                assert "b" in msg["data"]
            elif msg.get("topic") == "publicTrade.BTCUSDT":
                assert "topic" in msg
                assert "type" in msg
                assert "data" in msg
                assert "S" in msg["data"][0]
                assert "v" in msg["data"][0]
                assert "p" in msg["data"][0]
            if count >= 5000:
                break


async def test_private_stream():
    socket = BybitSocketManager(
        endpoint="private",
        api_key=os.getenv("BYBIT_API_KEY"),
        api_secret=os.getenv("BYBIT_API_SECRET"),
    )
    async with socket.connect():
        subscription = {
            "op": "subscribe",
            "args": ["order"],
        }
        await socket.subscribe(subscription)

        async for msg in socket.get_next_message():
            print(msg)



import hmac
import time
from contextlib import asynccontextmanager

import trio
import orjson
import trio_websocket
from trio_websocket import open_websocket_url

from trio_bybit.exceptions import BybitWebsocketOpError


class BybitSocketManager:
    URLS = {
        "main": {
            "spot": "wss://stream.bybit.com/v5/public/spot",
            "linear": "wss://stream.bybit.com/v5/public/linear",
            "inverse": "wss://stream.bybit.com/v5/public/inverse",
            "private": "wss://stream.bybit.com/v5/private",
        },
        "test": {
            "spot": "wss://stream-testnet.bybit.com/v5/public/spot",
            "linear": "wss://stream-testnet.bybit.com/v5/public/linear",
            "inverse": "wss://stream-testnet.bybit.com/v5/public/inverse",
            "private": "wss://stream-testnet.bybit.com/v5/private",
        },
        "demo": {
            "private": "wss://stream-demo.bybit.com",
        },
    }

    def __init__(
        self,
        endpoint: str = "spot",
        api_key: str | None = None,
        api_secret: str | None = None,
        alternative_net: str = "",
    ):
        self.ws: trio_websocket.WebSocketConnection | None = None
        self.endpoint: str = endpoint
        self.alternative_net: str = alternative_net if alternative_net else "main"
        if self.endpoint == "private" and (api_key is None or api_secret is None):
            raise ValueError("api_key and api_secret must be provided for private streams")
        self.api_key = api_key
        self.api_secret = api_secret
        self.conn_id: str | None = None

    @asynccontextmanager
    async def connect(self):
        try:
            url = self.URLS[self.alternative_net][self.endpoint]
        except KeyError:
            raise ValueError(f"endpoint {self.endpoint} with net {self.alternative_net} not supported")
        async with open_websocket_url(url) as ws:
            self.ws = ws
            async with trio.open_nursery() as nursery:
                nursery.start_soon(self.heartbeat)
                if self.endpoint == "private":
                    await self._send_signature()
                yield self.ws
                nursery.cancel_scope.cancel()

    async def heartbeat(self):
        while True:
            with trio.fail_after(5):
                await self.ws.send_message('{"op": "ping"}')
            await trio.sleep(20)

    async def _send_signature(self):
        expires = int((time.time() + 1) * 1000)
        signature = str(
            hmac.new(
                self.api_secret.encode("utf-8"), f"GET/realtime{expires}".encode("utf-8"), digestmod="sha256"
            ).hexdigest()
        )
        await self.ws.send_message(orjson.dumps({"op": "auth", "args": [self.api_key, expires, signature]}))
        auth_ret = orjson.loads(await self.ws.get_message())
        if auth_ret["op"] == "auth":
            assert auth_ret["success"]
            self.conn_id = auth_ret["conn_id"]

    async def subscribe(self, subscription: dict):
        """
        Subscribe or unsubscribe to a websocket stream.
        :param subscription: (un)subscription message, e.g. {"op": "subscribe", "args": ["publicTrade.BTCUSDT"]}
        """
        await self.ws.send_message(orjson.dumps(subscription))

    async def get_next_message(self):
        while True:
            raw_message = await self.ws.get_message()
            message = orjson.loads(raw_message)
            if "topic" in message and "data" in message:
                yield message
            elif "op" in message:
                if message["op"] == "pong":
                    continue
                if not message.get("success"):  # probably a subscription error
                    raise BybitWebsocketOpError(raw_message)



import os
import trio
import pytest_trio

from trio_bybit.client import AsyncClient


async def test_get_public():
    client = await AsyncClient.create()
    async with client:
        server_time = await client.get_server_time()
        assert isinstance(server_time, dict)
        assert server_time["retCode"] == 0
        assert server_time["retMsg"] == "OK"
        assert "timeSecond" in server_time["result"]
        assert "timeNano" in server_time["result"]
        assert "time" in server_time

        symbol_info = await client.get_symbol_info(category="linear", symbol="BTCUSDT")
        assert isinstance(symbol_info, dict)
        assert symbol_info["retCode"] == 0
        assert symbol_info["retMsg"] == "OK"
        assert symbol_info["result"]["category"] == "linear"
        assert symbol_info["result"]["list"][0]["symbol"] == "BTCUSDT"
        assert symbol_info["result"]["list"][0]["contractType"] == "LinearPerpetual"
        assert symbol_info["result"]["list"][0]["status"] == "Trading"
        assert symbol_info["result"]["list"][0]["baseCoin"] == "BTC"
        assert symbol_info["result"]["list"][0]["quoteCoin"] == "USDT"

        orderbook = await client.get_orderbook(category="linear", symbol="BTCUSDT")
        assert orderbook["retCode"] == 0
        assert orderbook["retMsg"] == "OK"


async def test_get_private():
    client = await AsyncClient.create(
        api_key=os.getenv("BYBIT_API_KEY"),
        api_secret=os.getenv("BYBIT_API_SECRET"),
        # alternative_net="demo",
    )

    async with client:
        wallet = await client.get_wallet_balance(accountType="UNIFIED")
        assert wallet["retCode"] == 0
        assert wallet["retMsg"] == "OK"
        assert wallet["result"]["list"][0]["accountType"] == "UNIFIED"
        position = await client.get_position_info(category="linear", settleCoin="USDT")
        assert position["retCode"] == 0
        assert position["retMsg"] == "OK"



import trio

from trio_bybit import AsyncClient


async def main():
    client = await AsyncClient.create(
        api_key='oJwzV0le5qsjkfslCf',
        api_secret='p2UKmTzwkkY4buKgr8ROr0SGhrp2sWd3XL2h',
    )
    order_args = {'category': 'spot', 'symbol': 'SOLUSDT', 'side': 'Buy', 'orderType': 'Limit',
                  'timeInForce': 'PostOnly', 'qty': '0.1', 'price': '134.99', 'marketUnit': 'baseCoin',
                  'orderLinkId': '14083-1714987999-be73914d-8e79-40c4-'}

    resp = await client.place_order(**order_args)

    print(resp)

    order_args = {'category': 'linear', 'symbol': 'SOLUSDT', 'side': 'Buy', 'orderType': 'Limit',
                  'timeInForce': 'PostOnly', 'qty': '0.1', 'price': '134.99', 'marketUnit': 'baseCoin',
                  'orderLinkId': '14083-1714987999-be73914d-8e79-40c5-'}

    resp = await client.place_order(**order_args)

    print(resp)


if __name__ == "__main__":
    trio.run(main)
